*** linux.old/net/tcp/arp.c	Thu Mar 11 20:30:03 1993
--- linux/net/tcp/arp.c	Thu Mar 11 20:44:55 1993
***************
*** 75,80 ****
--- 75,81 ----
  #define PRINTK(x) /**/
  #endif
  
+ static volatile num_arps=0;
  volatile struct arp_table *arp_table[ARP_TABLE_SIZE] ={NULL, };
  volatile struct sk_buff *arp_q=NULL;
  
***************
*** 313,326 ****
  }
  
  void
  arp_destroy(unsigned long paddr)
  {
    unsigned long hash;
    struct arp_table *apt;
    struct arp_table *lapt;
!   PRINTK (("arp_destroy (paddr=%X)\n",paddr));
!   /* we don't want to destroy are own arp */
!   if (my_ip_addr(paddr)) return;
    hash = net32(paddr) & (ARP_TABLE_SIZE - 1);
  
    cli(); /* can't be interrupted. */
--- 314,358 ----
  }
  
  void
+ arp_purge(void)
+ {
+   int i;
+ 
+   PRINTK (("arp_purge\n"));
+   cli();
+   for (i = 0; i < ARP_TABLE_SIZE; i++)
+   {
+     volatile struct arp_table *apt=arp_table[i];
+     struct arp_table *ptr=NULL;
+     struct arp_table *ptr2;
+ 
+     while (apt)
+       if (!apt->flag && before (apt->last_used, timer_seq+ARP_TIMEOUT))
+       {
+ 	if (ptr)
+ 	  ptr->next=apt->next;
+ 	else
+ 	  arp_table[i]=apt->next;
+ 	ptr2=apt;
+ 	apt=apt->next;
+         arp_free (ptr2, sizeof (*ptr2));
+       }
+       else
+       {
+         ptr=apt;
+         apt=apt->next;
+       }
+   }
+   sti();
+ }
+ 
+ void
  arp_destroy(unsigned long paddr)
  {
    unsigned long hash;
    struct arp_table *apt;
    struct arp_table *lapt;
!   PRINTK (("arp_destroy (paddr=%s)\n",in_ntoa(paddr)));
    hash = net32(paddr) & (ARP_TABLE_SIZE - 1);
  
    cli(); /* can't be interrupted. */
***************
*** 354,386 ****
    sti();
  }
  
- /* this routine does not check for duplicates.  It assumes the caller
-    does. */
- static  struct arp_table *
- create_arp (unsigned long paddr, unsigned char *addr, int hlen)
- {
-   struct arp_table *apt;
-   unsigned long hash;
-   apt = arp_malloc (sizeof (*apt), GFP_ATOMIC);
-   if (apt == NULL) return (NULL);
- 
-   hash = net32(paddr) & (ARP_TABLE_SIZE - 1);
-   apt->ip = paddr;
-   apt->hlen =hlen;
-   memcpy (apt->hard, addr, hlen);
-   apt->last_used=timer_seq;
-   cli();
-   apt->next = arp_table[hash];
-   arp_table[hash] = apt;
-   sti();
-   return (apt);
- }
- 
  int
  arp_rcv(struct sk_buff *skb, struct device *dev, struct packet_type *pt)
  {
     struct arp *arp;
-    struct arp_table *tbl;
     int ret;
  
     PRINTK (("<<\n"));
--- 386,395 ----
***************
*** 401,423 ****
         return (0);
      }
  
!   /* now look up the ip address in the table. */
!   tbl = arp_lookup (*arp_sourcep(arp));
!   if (tbl != NULL)
!     {
!        memcpy (tbl->hard, arp+1, arp->hlen);
!        tbl->hlen = arp->hlen;
!        tbl->last_used = timer_seq;
!     }
! 
    if (!my_ip_addr(*arp_targetp(arp)))
      {
         kfree_skb (skb, FREE_READ);
         return (0);
      }
! 
!   if (tbl == NULL)
!     create_arp (*arp_sourcep(arp), arp_sourceh(arp), arp->hlen);
  
     /* now see if we can send anything. */
     send_arp_q();
--- 410,424 ----
         return (0);
      }
  
!   PRINTK (("arp recieved (targetp= %X)\n", *arp_targetp(arp)));
!   
    if (!my_ip_addr(*arp_targetp(arp)))
      {
+        PRINTK (("arp not intended for us\n"));
         kfree_skb (skb, FREE_READ);
         return (0);
      }
!   arp_add(*arp_sourcep(arp),0, dev->addr_len, arp_sourceh(arp));
  
     /* now see if we can send anything. */
     send_arp_q();
***************
*** 439,451 ****
  {
    struct sk_buff *skb;
    struct arp *arp;
-   struct arp_table *apt;
    int tmp;
!   PRINTK (("arp_snd (paddr=%X, dev=%X, saddr=%X)\n",paddr, dev, saddr));
! 
!   /* first we build a dummy arp table entry. */
!   apt = create_arp (paddr, NULL, 0);
!   if (apt == NULL) return;
  
    skb = arp_malloc (sizeof (*arp) + sizeof (*skb) + dev->hard_header_len +
  		    2*dev->addr_len+8, GFP_ATOMIC);
--- 440,447 ----
  {
    struct sk_buff *skb;
    struct arp *arp;
    int tmp;
!   PRINTK (("arp_snd (paddr=%s, dev=%X, saddr=%s)\n",in_ntoa(paddr), dev, in_ntoa(saddr)));
  
    skb = arp_malloc (sizeof (*arp) + sizeof (*skb) + dev->hard_header_len +
  		    2*dev->addr_len+8, GFP_ATOMIC);
***************
*** 462,468 ****
    skb->next = NULL;
  
    tmp = dev->hard_header ((unsigned char *)(skb+1), dev,
! 			  ETHERTYPE_ARP, 0, saddr, skb->len);
    if (tmp < 0)
      {
         arp_free (skb->mem_addr, skb->mem_len);
--- 458,464 ----
    skb->next = NULL;
  
    tmp = dev->hard_header ((unsigned char *)(skb+1), dev,
! 			  ETHERTYPE_ARP, 0xffffffff, saddr, skb->len);
    if (tmp < 0)
      {
         arp_free (skb->mem_addr, skb->mem_len);
***************
*** 489,496 ****
  	   unsigned long saddr)
  {
    struct arp_table *apt;
!   PRINTK (("arp_find(haddr=%X, paddr=%X, dev=%X, saddr=%X)\n",
! 	  haddr, paddr, dev, saddr));
    if (my_ip_addr (paddr))
      {
        memcpy (haddr, dev->dev_addr, dev->addr_len);
--- 485,492 ----
  	   unsigned long saddr)
  {
    struct arp_table *apt;
!   PRINTK (("arp_find(haddr=%X, paddr=%s, dev=%X, saddr=%s)\n",
! 	  haddr, in_ntoa(paddr), dev, in_ntoa(saddr)));
    if (my_ip_addr (paddr))
      {
        memcpy (haddr, dev->dev_addr, dev->addr_len);
***************
*** 502,508 ****
         /* make sure it's not too old. If it is too old, we will
  	  just pretend we did not find it, and then arp_snd
  	  will verify the address for us. */
!        if (!before (apt->last_used, timer_seq+ARP_TIMEOUT) &&
  	   apt->hlen != 0)
  	 {
  	    apt->last_used=timer_seq;
--- 498,504 ----
         /* make sure it's not too old. If it is too old, we will
  	  just pretend we did not find it, and then arp_snd
  	  will verify the address for us. */
!        if ((apt->flag || !before (apt->last_used, timer_seq+ARP_TIMEOUT)) &&
  	   apt->hlen != 0)
  	 {
  	    apt->last_used=timer_seq;
***************
*** 523,547 ****
    return (1);
  }
  
! void
! arp_add (unsigned long addr, unsigned char *haddr, struct device *dev)
  {
!    struct arp_table *apt;
!    /* first see if the address is already in the table. */
!    apt = arp_lookup (addr);
!    if (apt != NULL)
!      {
! 	apt->last_used = timer_seq;
! 	memcpy (apt->hard, haddr , dev->addr_len);
! 	return;
!      }
!    create_arp (addr, haddr, dev->addr_len);
! }
  
! void
! arp_add_broad (unsigned long addr, struct device *dev)
! {
!   arp_add (addr,  dev->broadcast , dev);
  }
  
  void
--- 519,560 ----
    return (1);
  }
  
! int
! arp_add (unsigned long paddr, unsigned char flag, unsigned char hlen, unsigned char *haddr )
  {
!   unsigned long hash;
!   struct arp_table *apt;
!   PRINTK (("arp_add (paddr=%s)\n",in_ntoa(paddr)));
  
!   if (!paddr) return -1;
! 
!   hash = net32(paddr) & (ARP_TABLE_SIZE - 1);
!   cli();
!   for (apt = (struct arp_table *)arp_table[hash];
!        apt != NULL;
!        apt = (struct arp_table *)apt->next)
!       if (apt->ip == paddr)
! 	break;
!   if (apt == NULL)
!   {
!     if (num_arps < MAX_ARPS) arp_purge();
!     apt = arp_malloc (sizeof (*apt), GFP_ATOMIC);
!     if (apt == NULL)
!     {
!       sti();
!       return(-1);
!     }
!     num_arps++;
!     apt->next = arp_table[hash];
!     arp_table[hash] = apt;
!     apt->ip = paddr;
!   }
!   sti();
!   apt->flag = flag;
!   apt->hlen = hlen;
!   memcpy (apt->hard, haddr, hlen);
!   apt->last_used=timer_seq;
!   return(0);
  }
  
  void
***************
*** 585,607 ****
  int arp_ioctl_set(struct arpreq *req)
  {
  	struct arpreq		r;
- 	struct arp_table	*apt;
  
  	memcpy_fromfs(&r, req, sizeof(r));
  	if (arpreq_check(&r) != 0) return -EPFNOSUPPORT;
  
! 	apt = arp_lookup(*(unsigned long *)r.arp_pa.sa_data);
! 	if (apt) {
! 		apt->last_used = timer_seq;
! 		memcpy(apt->hard, r.arp_ha.sa_data , 6);
! 	} else {
! 		if (!create_arp(*(unsigned long *)r.arp_pa.sa_data,
! 				r.arp_ha.sa_data, 6)) {
! 			return -ENOMEM;
! 		}
! 	}
! 
! 	return 0;
  }
  
  int arp_ioctl_get(struct arpreq *req)
--- 598,611 ----
  int arp_ioctl_set(struct arpreq *req)
  {
  	struct arpreq		r;
  
  	memcpy_fromfs(&r, req, sizeof(r));
  	if (arpreq_check(&r) != 0) return -EPFNOSUPPORT;
  
! 	if (arp_add(*(unsigned long *)r.arp_pa.sa_data, r.arp_flags, 6, r.arp_ha.sa_data)==-1)
! 		return -ENOMEM;
! 	else
! 		return 0;
  }
  
  int arp_ioctl_get(struct arpreq *req)
***************
*** 614,619 ****
--- 618,624 ----
  	apt = arp_lookup(*(unsigned long *)r.arp_pa.sa_data);
  	if (apt) {
  		memcpy(r.arp_ha.sa_data, apt->hard, apt->hlen);
+ 		r.arp_flags=apt->flag;
  	} else {
  		return -ENXIO;
  	}
