Path: metallica.uwalt.nl.mugnet.org!utopia!hacktic!sun4nl!mcsun!uunet!munnari.oz.au!newshost.anu.edu.au!andosl!tridge
From: Andrew.Tridgell@anu.edu.au (Andrew Tridgell)
Newsgroups: comp.os.linux
Distribution: world
Subject: some patches for rlogin, rlogind and sock.c
Keywords: rlogin, rlogind, window size, SUNOS
NNTP-Posting-Host: 150.203.15.21
Message-ID: <1o3mg6INNs3a@manuel.anu.edu.au>
Organization: CSLab, Autralian National Uni.
Date: 16 Mar 1993 04:53:25 GMT
Lines: 230

Included below are some patches that address a couple of problems with
rlogin and rlogind.


1) rlogin to SunOS machines 

The first problem is that rlogin to SunOS machines produces an IO
error message and refuses to connect. This is caused by the fact that
rlogin only receives 1 SIGURG interrupt despite there being two pieces
of urgent data. I believe this is a problem with tcp.c but a work
around is to modify rlogin.c so it detects this condition and acts
accordingly (it send itself a SIGURG signal). There are also some
fixes to the signal handling in rlogin.c as it assumes BSD style
keep-the-handler behaviour but linux seems to have SYSV style
disconnect-the-handler behaviour. The fix should work with either
behaviour - it just does another signal() call at the end of each
interrupt handler.

When you fix the OOB handler (aka the urgent data handler) you must
also fix a ioctl problem as the BSD ioctl emulator in libbsd doesn't
seem to handle certain TIOCSETN calls. This is fixed in the patch by
removing that ioctl call (which was never used before as urgent data
never worked properly)


2) rlogind does not produce OOB data (window resizing problem)

when rlogind was ported to linux all the urgent data handling was
turned off - presumably because of the "rlogin to SunOS" bug. These
changes can now be undone and normal rlogind can be used as rlogin can
now handle urgent data (if you apply the above fix). The noticable
effect this has is that changing the size of an xterm will
automatically change the "rows" and "cols" stty values without user
intervention. This is especially useful when you resize a xterm
containing emacs - emacs will now automatically resize to fill the window.


3) sock.c closed/dead socket problem (rlogin to linux fails sometimes)

This was a very annoying problem to me. I use rlogin to get into my
linux box and after I have opened and closed a couple of windows from
machine A to my linux box I found opening any more was impossible from
A - but I could still open from another box.

The problem can be fixed by insering two lines in sock.c so that
sockets that are closed and dead are not returned by get_sock. They
seem to eventually be able to be re-used after some timer goes off -
so this is probably just a work around and not really fixing the
problem.



PATCH 1:

*** rlogin.c.original	Sat Mar 13 12:06:19 1993
--- rlogin.c	Tue Mar 16 15:44:05 1993
***************
*** 82,89 ****
--- 82,100 ----
  char dst_realm_buf[REALM_SZ], *dest_realm = NULL;
  extern char *krb_realmofhost();
  #endif
  
+ /*
+ * rlogin has problems with urgent data when logging into suns which
+ * results in the connection being closed with an IO error. SUN_KLUDGE
+ * is a work around - the actual bug is probably in tcp.c in the kernel, but
+ * I haven't managed to find it yet.
+ * Andrew.Tridgell@anu.edu.au (12th March 1993)
+ */
+ #ifdef linux
+ #define SUN_KLUDGE
+ #endif
+ 
  #ifndef TIOCPKT_WINDOW
  #define	TIOCPKT_WINDOW	0x80
  #endif
  
***************
*** 556,563 ****
--- 567,577 ----
  	    bcmp(&ws, &winsize, sizeof(ws))) {
  		winsize = ws;
  		sendwindow();
  	}
+ #ifdef SUN_KLUDGE
+        signal(SIGWINCH,sigwinch);
+ #endif
  }
  
  /*
   * Send the window size to the server via the magic escape
***************
*** 596,605 ****
  jmp_buf rcvtop;
  int ppid, rcvcnt, rcvstate;
  char rcvbuf[8 * 1024];
  
  void
! oob()
  {
  	struct sgttyb sb;
  	int atmark, n, out, rcvd;
  	char waste[BUFSIZ], mark;
--- 610,629 ----
  jmp_buf rcvtop;
  int ppid, rcvcnt, rcvstate;
  char rcvbuf[8 * 1024];
  
+ void oob()
+ {
+ void oob_real();
+ oob_real();
+ #ifdef SUN_KLUDGE
+ signal(SIGURG,oob);
+ #endif
+ }
+ 
+ 
  void
! oob_real()
  {
  	struct sgttyb sb;
  	int atmark, n, out, rcvd;
  	char waste[BUFSIZ], mark;
***************
*** 636,644 ****
--- 660,670 ----
  	if (!eight && (mark & TIOCPKT_NOSTOP)) {
  		(void)ioctl(0, TIOCGETP, (char *)&sb);
  		sb.sg_flags &= ~CBREAK;
  		sb.sg_flags |= RAW;
+ #ifndef SUN_KLUDGE
  		(void)ioctl(0, TIOCSETN, (char *)&sb);
+ #endif
  		notc.t_stopc = -1;
  		notc.t_startc = -1;
  		(void)ioctl(0, TIOCSETC, (char *)&notc);
  	}
***************
*** 728,735 ****
--- 754,778 ----
  		else
  #endif
  #endif
  			rcvcnt = read(rem, rcvbuf, sizeof (rcvbuf));
+ 
+ /*
+ * If we get a EIO from a read then it may mean that we have unread ungent data
+ * waiting that is getting in the way. We probably have got more then one lot of
+ * urgent data but we only got one SIGURG due to a problem in the kernel tcp.
+ * We can try and fix this by sending ourself a SIGURG and pretending the error
+ * never occurred. This might be a problem if we really _should_ be getting
+ * a EIO for some unrelated reason. (AJT 3/93)
+ */
+ #ifdef SUN_KLUDGE
+ 		if (rcvcnt < 0 && errno == EIO)
+ 		  {
+ 		    errno = 0;		    
+ 		    kill(getpid(),SIGURG);
+ 		    continue;
+ 		  }		
+ #endif    
  		if (rcvcnt == 0)
  			return (0);
  		if (rcvcnt < 0) {
  			if (errno == EINTR)
***************
*** 793,800 ****
--- 836,846 ----
  void
  copytochild()
  {
  	(void)kill(child, SIGURG);
+ #ifdef SUN_KLUDGE
+   signal(SIGCHLD,copytochild);
+ #endif
  }
  
  msg(str)
  	char *str;






PATCH 2:

Just remove all the "ifndef linux" bits from rlogind.c so the OOB
handling code is re-inserted.





PATCH 3:

Add the two lines marked with a "+" in get_sock() in sock.c

  for (s=prot->sock_array[num&(SOCK_ARRAY_SIZE-1)]; s != NULL; s=s->next)
    {
      if (s->num == num)
	{
	  /* we need to see if this is the socket that we want. */
	  if (!ip_addr_match (s->daddr, raddr))
	    continue;
	  if (s->dummy_th.dest != rnum && s->dummy_th.dest != 0)
	    continue;
	  if (!ip_addr_match (s->saddr, laddr))
	    continue;
+	  if (s->dead && (s->state == TCP_CLOSE))
+	    continue;
	  return (s);
	}
    }





Cheers,

Andrew

-- 
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
Andrew Tridgell                 CSLab, Research School of Physical Sciences
Andrew.Tridgell@anu.edu.au      Australian National University (x3064)
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

